@@ -4,16 +4,17 @@ Tests for stuff in django.utils.datastructures.
 
 import copy
 import pickle
-import warnings
 
 from django.test import SimpleTestCase
+from django.test.utils import IgnorePendingDeprecationWarningsMixin
 from django.utils.datastructures import (DictWrapper, ImmutableList,
     MultiValueDict, MultiValueDictKeyError, MergeDict, SortedDict)
 from django.utils import six
 
 
-class SortedDictTests(SimpleTestCase):
+class SortedDictTests(IgnorePendingDeprecationWarningsMixin, SimpleTestCase):
     def setUp(self):
+        super(SortedDictTests, self).setUp()
         self.d1 = SortedDict()
         self.d1[7] = 'seven'
         self.d1[1] = 'one'
@@ -134,34 +135,18 @@ class SortedDictTests(SimpleTestCase):
         self.assertEqual(list(reversed(self.d1)), [9, 1, 7])
         self.assertEqual(list(reversed(self.d2)), [7, 0, 9, 1])
 
-    def test_insert(self):
-        d = SortedDict()
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            d.insert(0, "hello", "world")
-        assert w[0].category is DeprecationWarning
 
-    def test_value_for_index(self):
-        d = SortedDict({"a": 3})
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            self.assertEqual(d.value_for_index(0), 3)
-        assert w[0].category is DeprecationWarning
-
-
-class MergeDictTests(SimpleTestCase):
+class MergeDictTests(IgnorePendingDeprecationWarningsMixin, SimpleTestCase):
 
     def test_simple_mergedict(self):
-        d1 = {'chris':'cool', 'camri':'cute', 'cotton':'adorable',
-              'tulip':'snuggable', 'twoofme':'firstone'}
+        d1 = {'chris': 'cool', 'camri': 'cute', 'cotton': 'adorable',
+              'tulip': 'snuggable', 'twoofme': 'firstone'}
 
-        d2 = {'chris2':'cool2', 'camri2':'cute2', 'cotton2':'adorable2',
-              'tulip2':'snuggable2'}
+        d2 = {'chris2': 'cool2', 'camri2': 'cute2', 'cotton2': 'adorable2',
+              'tulip2': 'snuggable2'}
 
-        d3 = {'chris3':'cool3', 'camri3':'cute3', 'cotton3':'adorable3',
-              'tulip3':'snuggable3'}
-
-        d4 = {'twoofme': 'secondone'}
+        d3 = {'chris3': 'cool3', 'camri3': 'cute3', 'cotton3': 'adorable3',
+              'tulip3': 'snuggable3'}
 
         md = MergeDict(d1, d2, d3)
 
@@ -194,14 +179,17 @@ class MergeDictTests(SimpleTestCase):
 
         self.assertTrue('value1' in six.itervalues(mm))
 
-        self.assertEqual(sorted(six.iteritems(mm), key=lambda k: k[0]),
-                          [('key1', 'value1'), ('key2', 'value3'),
-                           ('key4', 'value6')])
+        self.assertEqual(
+            sorted(six.iteritems(mm), key=lambda k: k[0]),
+            [('key1', 'value1'), ('key2', 'value3'), ('key4', 'value6')]
+        )
 
-        self.assertEqual([(k,mm.getlist(k)) for k in sorted(mm)],
-                          [('key1', ['value1']),
-                           ('key2', ['value2', 'value3']),
-                           ('key4', ['value5', 'value6'])])
+        self.assertEqual(
+            [(k, mm.getlist(k)) for k in sorted(mm)],
+            [('key1', ['value1']),
+             ('key2', ['value2', 'value3']),
+             ('key4', ['value5', 'value6'])]
+        )
 
     def test_bool_casting(self):
         empty = MergeDict({}, {}, {})
@@ -227,12 +215,15 @@ class MultiValueDictTests(SimpleTestCase):
         self.assertEqual(d['name'], 'Simon')
         self.assertEqual(d.get('name'), 'Simon')
         self.assertEqual(d.getlist('name'), ['Adrian', 'Simon'])
-        self.assertEqual(sorted(list(six.iteritems(d))),
-                          [('name', 'Simon'), ('position', 'Developer')])
+        self.assertEqual(
+            sorted(list(six.iteritems(d))),
+            [('name', 'Simon'), ('position', 'Developer')]
+        )
 
-        self.assertEqual(sorted(list(six.iterlists(d))),
-                          [('name', ['Adrian', 'Simon']),
-                           ('position', ['Developer'])])
+        self.assertEqual(
+            sorted(list(six.iterlists(d))),
+            [('name', ['Adrian', 'Simon']), ('position', ['Developer'])]
+        )
 
         six.assertRaisesRegex(self, MultiValueDictKeyError, 'lastname',
             d.__getitem__, 'lastname')
@@ -313,5 +304,7 @@ class DictWrapperTests(SimpleTestCase):
     def test_dictwrapper(self):
         f = lambda x: "*%s" % x
         d = DictWrapper({'a': 'a'}, f, 'xx_')
-        self.assertEqual("Normal: %(a)s. Modified: %(xx_a)s" % d,
-                          'Normal: a. Modified: *a')
+        self.assertEqual(
+            "Normal: %(a)s. Modified: %(xx_a)s" % d,
+            'Normal: a. Modified: *a'
+        )
